import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * Eine grafische Benutzeroberflaeche zur Verwaltung von Klausuren.
 *
 * @author Annabelle Klarl
 */
public class KlausurverwaltungsFrame extends JFrame implements ActionListener {

  private JButton klausurAnlegenButton;
  private JButton studentAnmeldenButton;
  private JButton studentSuchenButton;
  private JButton noteEintragenButton;

  /* Hausaufgabe */
  private JButton durchschnittsnoteButton;
  private JButton studentenProNoteButton;

  private JTextArea ausgabeBereich;

  private Klausur aktuelleKlausur;

  /** In diesem Programmstueck wird das Fenster erzeugt. */
  public KlausurverwaltungsFrame() {
    /* In der Kopfleiste des Fenster steht "KlausurverwaltungFrame" */
    this.setTitle("KlausurverwaltungFrame");

    /* Das Fenster ist 700 Pixel breit und 350 Pixel hoch. */
    this.setSize(700, 350);

    /* Hier werden alle Buttons erzeugt. */
    this.klausurAnlegenButton = new JButton("Lege Klausur an");
    this.studentAnmeldenButton = new JButton("Student anmelden");
    this.studentSuchenButton = new JButton("Student suchen");
    this.noteEintragenButton = new JButton("Note eintragen");

    /* Hausaufgabe */
    this.durchschnittsnoteButton = new JButton("Durchschnittsnote berechnen");
    this.studentenProNoteButton =
        new JButton("Anzahl an Studenten mit Note x anzeigen");

    /* Hier wird der Ausgabe-Bereich erzeugt. */
    this.ausgabeBereich = new JTextArea(10, 100);

    /* Hier werden alle Buttons zusammengruppiert. */
    JPanel studentenPanel = new JPanel();
    studentenPanel.setLayout(new GridLayout(5, 1));
    studentenPanel.add(this.studentAnmeldenButton);
    studentenPanel.add(this.studentSuchenButton);
    studentenPanel.add(this.noteEintragenButton);

    /* Hausaufgabe */
    studentenPanel.add(this.durchschnittsnoteButton);
    studentenPanel.add(this.studentenProNoteButton);

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1, 2));
    buttonPanel.add(this.klausurAnlegenButton);
    buttonPanel.add(studentenPanel);

    /*
     * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
     * Interaktionselemente (wie eine TextArea oder ein Button) platziert
     * werden koennen.
     */
    Container contentPane = this.getContentPane();
    contentPane.setLayout(new GridLayout(2, 1));
    /* Hier wird die Gruppe von Buttons platziert. */
    contentPane.add(buttonPanel);
    /* Hier wird der Ausgabebereich platziert. */
    contentPane.add(this.ausgabeBereich);

    /*
     * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei jedem
     * Button registriert.
     */
    this.klausurAnlegenButton.addActionListener(this);
    this.studentAnmeldenButton.addActionListener(this);
    this.studentSuchenButton.addActionListener(this);
    this.noteEintragenButton.addActionListener(this);

    /*
     * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
     * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
     */
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }

  /**
   * Dieses Programmstueck wird immer dann ausgefuehrt,
   * wenn ein Benutzer auf einen Button drueckt.
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
    if (source == this.klausurAnlegenButton) {
      this.klausurAnlegen();
    } else if (source == this.studentAnmeldenButton) {
      this.studentAnmelden();
    } else if (source == this.studentSuchenButton) {
      this.studentSuchen();
    } else if (source == this.noteEintragenButton) {
      this.noteEintragen();
    }
  }

  /** Diese Methode erzeugt eine neue Klausur, falls es momentan keine gibt. */
  private void klausurAnlegen() {
    if (this.aktuelleKlausur != null) {
      this.ausgabeBereich.setText(
          "Es wurde bereits eine Klausur mit dem Titel "
              + this.aktuelleKlausur.getTitel()
              + " angelegt.");
    } else {
      String einlesenTitel = JOptionPane.showInputDialog("Titel der Klausur: ");

      String einlesenMaxTeilnehmer =
          JOptionPane.showInputDialog("Maximale Anzahl an Teilnehmern: ");
      int maxTeilnehmer = Integer.parseInt(einlesenMaxTeilnehmer);

      // !!!!
      this.aktuelleKlausur = new Klausur(einlesenTitel, maxTeilnehmer);

      this.ausgabeBereich.setText(
          "Folgende Klausur wurde angelegt: "
              + this.aktuelleKlausur
              + " mit dem Titel "
              + this.aktuelleKlausur.getTitel());
    }
  }

  /**
   * Diese Methode meldet einen Studenten zur Klausur an,
   * falls es momentan eine Klausur gibt.
   */
  private void studentAnmelden() {
    if (this.aktuelleKlausur == null) {
      this.ausgabeBereich.setText("Es wurde noch keine Klausur angelegt.");
    } else {
      String einlesenMatrNr = JOptionPane.showInputDialog("Matrikelnummer: ");
      int matrNr = Integer.parseInt(einlesenMatrNr);

      String name = JOptionPane.showInputDialog("Name: ");

      // !!! Methodenaufruf
      boolean angemeldet = this.aktuelleKlausur.anmelden(matrNr, name);
      if (angemeldet) {
        this.ausgabeBereich.setText(
            "Der Student "
                + name
                + " mit der Matrikelnummer "
                + matrNr
                + " wurde angemeldet.");
      } else {
        this.ausgabeBereich.setText(
            "Der Student konnte nicht angemeldet werden, "
                + "da die Maximalanzahl an Teilnehmern "
                + "ueberschritten wurde.");
      }
    }
  }

  /**
   * Diese Methode gibt Informationen ueber einen an der Klausur angemeldeten
   * Studenten aus, falls es momentan eine Klausur gibt und ein Student mit
   * der gegebenen Matrikelnummer angemeldet ist.
   */
  private void studentSuchen() {
    if (this.aktuelleKlausur == null) {
      this.ausgabeBereich.setText("Es wurde noch keine Klausur angelegt.");
    } else {
      String einlesenMatrNr = JOptionPane.showInputDialog("Matrikelnummer: ");
      int matrNr = Integer.parseInt(einlesenMatrNr);

      // !!! Methodenaufruf
      Student student = this.aktuelleKlausur.sucheStudent(matrNr);
      if (student == null) {
        this.ausgabeBereich.setText(
            "Der Student konnte nicht gefunden werden.");
      } else {
        this.ausgabeBereich.setText(
            "Folgender Student wurde gefunden: "
                + student.getName()
                + " mit der Matrikelnummer "
                + student.getMatrNr()
                + " und der Note "
                + student.getNote());
      }
    }
  }

  /**
   * Diese Methode traegt eine Note fuer einen Studenten ein, falls es
   * momentan eine Klausur gibt und ein Student mit der gegebene
   * Matrikelnummer angemeldet ist.
   */
  private void noteEintragen() {
    if (this.aktuelleKlausur == null) {
      this.ausgabeBereich.setText("Es wurde noch keine Klausur angelegt.");
    } else {
      String einlesenMatrNr = JOptionPane.showInputDialog("Matrikelnummer: ");
      int matrNr = Integer.parseInt(einlesenMatrNr);

      String einlesenNote = JOptionPane.showInputDialog("Note: ");
      double note = Double.parseDouble(einlesenNote);

      // !!! Methodenaufruf
      boolean benotet = this.aktuelleKlausur.noteEintragen(matrNr, note);
      if (benotet) {
        this.ausgabeBereich.setText(
            "Der Student mit der Matrikelnummer "
                + matrNr
                + " wurde mit der Note "
                + note
                + " benotet.");
      } else {
        this.ausgabeBereich.setText(
            "Der Student konnte nicht benotet werden, "
                + "da kein Student mit der Matrikelnummer "
                + matrNr
                + " gefunden werden konnte.");
      }
    }
  }
}
